/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.config.gui;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;
import org.apache.jmeter.config.*;

import org.apache.jmeter.gui.*;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    March 13, 2001
 *@version    1.0
 ***********************************************************/

public class ArgumentsPanel extends JPanel implements ModelSupported, KeyListener,
		ActionListener
{
	JTable table;
	Arguments model;
	JButton add;
	JButton delete;

	/************************************************************
	 *  Constructor for the ArgumentsPanel object
	 ***********************************************************/
	public ArgumentsPanel()
	{
	}

	/************************************************************
	 *  Sets the Model attribute of the ArgumentsPanel object
	 *
	 *@param  model  The new Model value
	 ***********************************************************/
	public void setModel(Object model)
	{
		this.model = (Arguments)model;
		init();
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void updateGui()
	{
		model.fireTableDataChanged();
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyReleased(KeyEvent e)
	{
		model.setValueAt(table.getValueAt(table.getSelectedRow(), table.getSelectedColumn()),
				table.getSelectedRow(), table.getSelectedColumn());
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyPressed(KeyEvent e)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void keyTyped(KeyEvent e)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void actionPerformed(ActionEvent e)
	{
		String action = e.getActionCommand();
		if(action.equals("Delete"))
		{
			model.removeRow(table.getSelectedRow());
			model.fireTableDataChanged();
		}
		else if(action.equals("Add"))
		{
			model.addNewRow();
			model.fireTableDataChanged();
		}
	}

	private void init()
	{
		this.setLayout(new VerticalLayout());
		this.add(new JLabel("Parameter Table"));
		table = new JTable(model);
		table.addKeyListener(this);
		table.setCellSelectionEnabled(true);
		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JScrollPane scroller = new JScrollPane(table);
		Dimension tableDim = scroller.getPreferredSize();
		tableDim.height = (int)tableDim.getHeight() / 2;
		scroller.setPreferredSize(tableDim);
		scroller.setMaximumSize(tableDim);

		scroller.setColumnHeaderView(table.getTableHeader());
		this.add(scroller);
		add = new JButton("Add");
		delete = new JButton("Delete");
		JPanel buttonPanel = new JPanel();
		add.addActionListener(this);
		delete.addActionListener(this);
		buttonPanel.add(add);
		buttonPanel.add(delete);
		this.add(buttonPanel);
	}

}
