/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.gui;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;

import org.apache.jmeter.gui.action.*;
import org.apache.jmeter.util.JMeterUtils;

/**
 * Title:        Apache JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache Foundation
 * @author Michael Stover
 * @version 1.0
 */

public class JMeterMenuBar extends JMenuBar
{
	JMenu fileMenu;
		JMenuItem file_save;
		JMenuItem file_save_all;
		JMenuItem file_load;
		JMenuItem file_exit;
	JMenu runMenu;
			JMenuItem run_start;
			JMenu remote_start;
					Collection remote_engine_start;
			JMenuItem run_stop;
			JMenu remote_stop;
					Collection remote_engine_stop;
			JMenuItem run_clear;
	JMenu reportMenu;
			JMenuItem analyze;
	JMenu optionsMenu;
			JMenu lafMenu;
	JMenu helpMenu;
			JMenuItem help_about;

	String[] remoteHosts;

	public JMeterMenuBar()
	{
		remote_engine_start = new LinkedList();
		remote_engine_stop = new LinkedList();
		remoteHosts = JMeterUtils.split(
					JMeterUtils.getPropDefault("remote_hosts",""),",");
		if(remoteHosts.length == 1 && remoteHosts[0].equals(""))
		{
			remoteHosts = new String[0];
		}
		this.getRemoteItems();

		createMenuBar();
	}

	public void setSaveEnabled(boolean save)
	{
		file_save.setEnabled(save);
		file_save_all.setEnabled(save);
	}


	/************************************************************
	 *  Creates the MenuBar for this application. I believe in my heart that this
	 *  should be defined in a file somewhere, but that is for later.
	 ***********************************************************/
	public void createMenuBar()
	{

		fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		file_save = new JMenuItem("Save",'S');
		file_save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,KeyEvent.CTRL_MASK));
		file_save.addActionListener(ActionRouter.getInstance());
		file_save_all = new JMenuItem("Save All",'A');
		file_save_all.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,KeyEvent.CTRL_MASK));
		file_save_all.addActionListener(ActionRouter.getInstance());

		file_load = new JMenuItem("Open",'O');
		file_load.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,KeyEvent.CTRL_MASK));
		file_load.addActionListener(ActionRouter.getInstance());

		file_exit = new JMenuItem("Exit", 'X');
		file_exit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, KeyEvent.CTRL_MASK));
		file_exit.setActionCommand("exit");
		file_exit.addActionListener(ActionRouter.getInstance());
		fileMenu.add(file_load);
		fileMenu.add(file_save);
		fileMenu.add(file_save_all);
		fileMenu.add(file_exit);

		runMenu = new JMenu("Run");
		runMenu.setMnemonic('R');
		run_start = new JMenuItem("Start", 'S');
		run_start.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_MASK));
		run_start.addActionListener(ActionRouter.getInstance());
		run_start.setActionCommand("start");
		run_stop = new JMenuItem("Stop", 'T');
		run_stop.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PERIOD, KeyEvent.CTRL_MASK));
		run_stop.setEnabled(false);
		run_stop.addActionListener(ActionRouter.getInstance());
		run_stop.setActionCommand("stop");
		run_clear = new JMenuItem("Clear", 'C');
		run_clear.addActionListener(ActionRouter.getInstance());
		run_clear.setActionCommand("clear");
		runMenu.add(run_start);
		if(remote_start != null)
		{
			runMenu.add(remote_start);
		}
		runMenu.add(run_stop);
		if(remote_stop != null)
		{
			runMenu.add(remote_stop);
		}
		runMenu.add(run_clear);

		reportMenu = new JMenu("Report");
		reportMenu.setMnemonic('P');
		analyze = new JMenuItem("Analyze Data File...");
		analyze.setMnemonic('A');
		analyze.addActionListener(ActionRouter.getInstance());
		reportMenu.add(analyze);

		optionsMenu = new JMenu("Options");
		lafMenu = new JMenu("Look and Feel");
		UIManager.LookAndFeelInfo lafs[] = UIManager.getInstalledLookAndFeels();
		for (int i = 0; i < lafs.length; ++i)
		{
			JMenuItem laf = new JMenuItem(lafs[i].getName());
			laf.addActionListener(ActionRouter.getInstance());
			laf.setActionCommand("laf:" + lafs[i].getClassName());
			lafMenu.add(laf);
		}
		optionsMenu.setMnemonic('O');
		optionsMenu.add(lafMenu);

		helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');
		help_about = new JMenuItem("About", 'A');
		help_about.setActionCommand("about");
		help_about.addActionListener(ActionRouter.getInstance());
		helpMenu.add(help_about);

		this.add(fileMenu);
		this.add(runMenu);
		this.add(reportMenu);
		this.add(optionsMenu);
		// This makes the help stick to the right side
		// I stole this from the java.sun.com tutorial page
		this.add(Box.createHorizontalGlue());
		this.add(helpMenu);
	}

	private void getRemoteItems()
	{
		if(remoteHosts.length > 0)
		{
			remote_start = new JMenu("Remote Start");
			remote_stop = new JMenu("Remote Stop");
			for (int i = 0; i < remoteHosts.length; i++)
			{
				JMenuItem item = new JMenuItem(remoteHosts[i]);
				item.setActionCommand("remote_start");
				item.setName(remoteHosts[i]);
				item.addActionListener(ActionRouter.getInstance());
				remote_engine_start.add(item);
				remote_start.add(item);

				item = new JMenuItem(remoteHosts[i]);
				item.setActionCommand("remote_stop");
				item.setName(remoteHosts[i]);
				item.addActionListener(ActionRouter.getInstance());
				remote_engine_stop.add(item);
				remote_stop.add(item);
			}
		}
	}

	public void setRunning(boolean running,String host)
	{
		Iterator iter = remote_engine_start.iterator();
		Iterator iter2 = remote_engine_stop.iterator();
		while (iter.hasNext() && iter2.hasNext())
		{
			((JMenuItem)iter.next()).setEnabled(!running);
			((JMenuItem)iter2.next()).setEnabled(running);
		}
	}


	public void setEnabled(boolean enable)
	{
		run_start.setEnabled(!enable);
		run_stop.setEnabled(enable);
	}
}