/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.gui.tree;

import javax.swing.tree.*;

import java.util.*;

import org.apache.jmeter.threads.ThreadGroup;
import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.control.*;
import org.apache.jmeter.timers.Timer;
import org.apache.jmeter.samplers.SampleListener;
import org.apache.jmeter.gui.JMeterComponentModel;
import org.apache.jmeter.util.ListedHashTree;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class JMeterTreeModel extends DefaultTreeModel
{


	public JMeterTreeModel()
	{
		super(new JMeterTreeNode(new WorkBench("Root")));
			this.insertNodeInto(new JMeterTreeNode(TestPlan.createTestPlan("Test Plan")),
					(JMeterTreeNode)getRoot(),0);
			this.insertNodeInto(new JMeterTreeNode(new WorkBench("WorkBench")),
					(JMeterTreeNode)getRoot(),1);
	}

	public void addSubTree(ListedHashTree subTree,JMeterTreeNode current)
	{
		Iterator iter = subTree.list().iterator();
		while (iter.hasNext())
		{
			JMeterComponentModel item = (JMeterComponentModel)iter.next();
			if(item instanceof TestPlan)
			{
				current = (JMeterTreeNode)((JMeterTreeNode)getRoot()).getChildAt(0);
				addSubTree(subTree.get(item),current);
			}
			else
			{
				addSubTree(subTree.get(item),addComponent(item,current));
			}
		}
	}

	public JMeterTreeNode addComponent(JMeterComponentModel component,JMeterTreeNode node)
	{
		JMeterTreeNode newNode = new JMeterTreeNode(component);
		this.insertNodeInto(newNode,node,node.getChildCount());
		return newNode;
	}

	public JMeterComponentModel compileComponent(JMeterTreeNode node)
	{
		traverseNode(node);
		return (JMeterComponentModel)node.getUserObject();
	}

	private void traverseNode(JMeterTreeNode node)
	{
		node.uncompile();
		Enumeration enum = node.children();
		while (enum.hasMoreElements())
		{
			JMeterTreeNode child = (JMeterTreeNode)enum.nextElement();
			traverseNode(child);
		}
		JMeterTreeNode parent = (JMeterTreeNode)node.getParent();
		addChildToParent(node.getUserObject(),parent.getUserObject());
	}

	private void addChildToParent(Object child,Object parent)
	{
		if(parent instanceof ThreadGroup)
		{
			if(child instanceof Timer)
			{
				((ThreadGroup)parent).addTimer((Timer)child);
			}
			else if(child instanceof ConfigElement)
			{
				((ThreadGroup)parent).addConfigElement((ConfigElement)child);
			}
			else if(child instanceof SamplerController)
			{
				((ThreadGroup)parent).addSamplerController((SamplerController)child);
			}
			else if(child instanceof SampleListener)
			{
				((ThreadGroup)parent).addListener((SampleListener)child);
			}
		}
		else if(parent instanceof SamplerController)
		{
			if(child instanceof SamplerController)
			{
				((SamplerController)parent).addSamplerController((SamplerController)child);
			}
			else if(child instanceof ConfigElement)
			{
				((SamplerController)parent).addConfigElement((ConfigElement)child);
			}
		}
		else if(parent instanceof TestPlan)
		{
			if(child instanceof ThreadGroup)
			{
				((TestPlan)parent).addThreadGroup((ThreadGroup)child);
			}
		}
	}
}