/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.ftp.config.gui;

import javax.swing.*;
import java.awt.event.*;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.protocol.ftp.config.*;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class FtpConfigGui extends JPanel implements ModelSupported,KeyListener
{

	private static final String SERVER = "server";
	private static final String FILENAME = "filename";

	private FtpConfig model;

	private NamePanel namePanel;
	private JTextField server = new JTextField(30);
	private JTextField filename = new JTextField(30);

	private boolean displayName = true;

	public FtpConfigGui()
	{
	}

	public FtpConfigGui(boolean displayName)
	{
		this.displayName = displayName;
	}

	public void setModel(Object model)
	{
		this.model = (FtpConfig)model;
		init();
	}

	public void updateGui()
	{
		if(displayName)
		{
			namePanel.updateGui();
		}
		server.setText(model.getServer());
		filename.setText(model.getFilename());
	}

	private void init()
	{
		this.setLayout(new VerticalLayout(1,VerticalLayout.LEFT));
		if(displayName)
		{
			namePanel = new NamePanel(model);
			this.add(namePanel);
		}

		JPanel serverPanel = new JPanel();
		serverPanel.add(new JLabel("Server"));
		server.addKeyListener(this);
		server.setText(model.getServer());
		server.setName(SERVER);
		serverPanel.add(server);
		this.add(serverPanel);

		JPanel filenamePanel = new JPanel();
		filenamePanel.add(new JLabel("Filename"));
		filename.addKeyListener(this);
		filename.setName(FILENAME);
		filename.setText(model.getServer());
		filenamePanel.add(filename);
		this.add(filenamePanel);
	}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyTyped(KeyEvent e)
	{
	}

	public void keyReleased(KeyEvent e)
	{
		String name = e.getComponent().getName();
		if(name.equals(SERVER))
		{
			model.setServer(server.getText());
		}
		else if(name.equals(FILENAME))
		{
			model.setFilename(filename.getText());
		}
	}


}