/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.http.control;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.*;

import org.apache.jmeter.gui.JMeterComponentModel;
import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.save.Saveable;
/************************************************************
 *  This class provides a way to provide Authorization in jmeter requests. The
 *  format of the authorization file is: URL user pass where URL is an HTTP URL,
 *  user a username to use and pass the appropriate password.
 *
 *@author     <a href="mailto:luta.raphael@networks.vivendi.com">Raphal Luta
 *      </a>
 *@created    $Date: 2001/03/17 22:25:49 $
 *@version    0.9
 ***********************************************************/
public class AuthManager extends AbstractTableModel implements ConfigElement, JMeterComponentModel,Saveable
{
	/************************************************************
	 *  A vector of Authorization objects managed by this class.
	 *
	 *@associates             <{org.apache.jmeter.samplers.Authorization}>
	 *@supplierCardinality    0..
	 ***********************************************************/
	private Vector auths = new Vector();
	private String name;

	private final static int columnCount = 3;
	private final static String[] columnNames = {"Base URL", "Username", "Password"};

	/************************************************************
	 *  Default Constructor
	 ***********************************************************/
	public AuthManager()
	{
	}

	public void addConfigElement(ConfigElement config)
	{
	}

	public void addAuth(Authorization auth)
	{
		auths.add(auth);
	}

	public Collection getAuthObjects()
	{
		return auths;
	}

	public boolean expectsModification()
	{
		return false;
	}

	public Object clone()
	{
		try
		{
			return super.clone();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		return null;
	}

	public void uncompile()
	 {
	 }

	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.protocol.http.save.AuthManagerHandler.class;
	}

	/************************************************************
	 *  update an authentication record
	 *
	 *@param  index  !ToDo (Parameter description)
	 *@param  url    !ToDo (Parameter description)
	 *@param  user   !ToDo (Parameter description)
	 *@param  pass   !ToDo (Parameter description)
	 ***********************************************************/
	public void set(int index, String url, String user, String pass)
	{
		Authorization auth = new Authorization(url, user, pass);
		if (index >= 0)
		{
			auths.setElementAt(auth, index);
		}
		else
		{
			auths.addElement(auth);
		}
		fireTableChanged(new TableModelEvent(this));
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  newName  !ToDo (Parameter description)
	 ***********************************************************/
	public void setName(String newName)
	{
		name = newName;
	}

	/************************************************************
	 *  required by table model interface
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public int getRowCount()
	{
		return auths.size();
	}

	/************************************************************
	 *  required by table model interface
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public int getColumnCount()
	{
		return columnCount;
	}

	/************************************************************
	 *  required by table model interface
	 *
	 *@param  column  !ToDo (Parameter description)
	 *@return         !ToDo (Return description)
	 ***********************************************************/
	public String getColumnName(int column)
	{
		return columnNames[column];
	}

	/************************************************************
	 *  required by table model interface
	 *
	 *@param  column  !ToDo (Parameter description)
	 *@return         !ToDo (Return description)
	 ***********************************************************/
	public Class getColumnClass(int column)
	{
		return columnNames[column].getClass();
	}

	/************************************************************
	 *  required by table model interface
	 *
	 *@param  row     !ToDo (Parameter description)
	 *@param  column  !ToDo (Parameter description)
	 *@return         !ToDo (Return description)
	 ***********************************************************/
	public Object getValueAt(int row, int column)
	{
		Authorization auth = (Authorization)auths.elementAt(row);
		if (column == 0)
		{
			return auth.getURL();
		}
		else if (column == 1)
		{
			return auth.getUser();
		}
		else if (column == 2)
		{
			return auth.getPass();
		}
		return null;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getClassLabel()
	{
		return "Http Authorization";
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.protocol.http.gui.AuthPanel.class;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection getAddList()
	{
		return null;
	}

	/************************************************************
	 *  return the record at index i
	 *
	 *@param  i  !ToDo (Parameter description)
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Authorization get(int i)
	{
		return (Authorization)auths.elementAt(i);
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@param  url  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	public String getAuthHeaderForURL(URL url)
	{
		if (!url.getProtocol().toUpperCase().equals("HTTP"))
		{
			return null;
		}

		StringBuffer header = new StringBuffer();
		for (Enumeration enum = auths.elements(); enum.hasMoreElements(); )
		{
			Authorization auth = (Authorization)enum.nextElement();
			if (url.toString().startsWith(auth.getURL()))
			{
				header.append("Basic " + Base64Encoder.encode(auth.getUser() + ":" + auth.getPass()));
				break;
			}
		}

		if (header.length() != 0)
		{
			return header.toString();
		}
		else
		{
			return null;
		}
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	/************************************************************
	 *  save the authentication data to a file
	 *
	 *@param  authFile         !ToDo (Parameter description)
	 *@exception  IOException  !ToDo (Exception description)
	 ***********************************************************/
	public void save(String authFile) throws IOException
	{
		File file = new File(authFile);
		if (!file.isAbsolute())
		{
			file = new File(System.getProperty("user.dir") + File.separator + authFile);
		}
		PrintWriter writer = new PrintWriter(new FileWriter(file));
		writer.println("# JMeter generated Authorization file");
		for (int i = 0; i < auths.size(); i++)
		{
			Authorization auth = (Authorization)auths.elementAt(i);
			writer.println(auth.toString());
		}
		writer.flush();
		writer.close();
	}

	/************************************************************
	 *  add authentication data from a file
	 *
	 *@param  authFile         !ToDo
	 *@exception  IOException  !ToDo (Exception description)
	 ***********************************************************/
	public void addFile(String authFile) throws IOException
	{
		File file = new File(authFile);
		if (!file.isAbsolute())
		{
			file = new File(System.getProperty("user.dir") + File.separator + authFile);
		}
		BufferedReader reader = null;
		if (file.canRead())
		{
			reader = new BufferedReader(new FileReader(file));
		}
		else
		{
			JOptionPane.showMessageDialog(null, "The file you specified cannot be read.",
					"Information", JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		String line;
		while ((line = reader.readLine()) != null)
		{
			try
			{
				if (line.startsWith("#") || line.trim().length() == 0)
				{
					continue;
				}
				StringTokenizer st = new StringTokenizer(line, "\t");
				String url = st.nextToken();
				String user = st.nextToken();
				String pass = st.nextToken();
				Authorization auth = new Authorization(url, user, pass);
				auths.addElement(auth);
			}
			catch (Exception e)
			{
				throw new IOException("Error parsing auth line\n\t'" + line + "'\n\t" + e);
			}
		}
		reader.close();
		fireTableChanged(new TableModelEvent(this));
	}

	/************************************************************
	 *  remove an authentication record
	 *
	 *@param  index  !ToDo (Parameter description)
	 ***********************************************************/
	public void remove(int index)
	{
		auths.removeElementAt(index);
		fireTableChanged(new TableModelEvent(this));
	}

	/************************************************************
	 *  return the number of records
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public int size()
	{
		return auths.size();
	}
}

/************************************************************
 *  This class provides an implementation of Base64 encoding without relying on
 *  the the sun.* packages.
 *
 *@author     $Author: mstover1 $
 *@created    $Date: 2001/03/17 22:25:49 $
 *@version    $Revision: 1.7 $
 ***********************************************************/

class Base64Encoder
{

	private final static char[] pem_array = {65, 66, 67, 68, 69, 70, 71, 72, 73, 74,
			75, 76, 77, 78, 79, 80, 81, 82, 83, 84,
			85, 86, 87, 88, 89, 90, 97, 98, 99, 100,
			101, 102, 103, 104, 105, 106, 107, 108,
			109, 110, 111, 112, 113, 114, 115, 116,
			117, 118, 119, 120, 121, 122, 48, 49, 50,
			51, 52, 53, 54, 55, 56, 57, 43, 47};
	private final static char eq = 61;

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public Base64Encoder()
	{
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  s  !ToDo (Parameter description)
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public final static String encode(String s)
	{
		return encode(s.getBytes());
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  bs  !ToDo (Parameter description)
	 *@return     !ToDo (Return description)
	 ***********************************************************/
	public final static String encode(byte[] bs)
	{
		StringBuffer out = new StringBuffer("");
		int bl = bs.length;
		for (int i = 0; i < bl; i += 3)
		{
			out.append(encodeAtom(bs, i, (bl - i)));
		}
		return out.toString();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  b     !ToDo (Parameter description)
	 *@param  strt  !ToDo (Parameter description)
	 *@param  left  !ToDo (Parameter description)
	 *@return       !ToDo (Return description)
	 ***********************************************************/
	public final static String encodeAtom(byte[] b, int strt, int left)
	{
		StringBuffer out = new StringBuffer("");
		if (left == 1)
		{
			byte b1 = b[strt];
			int k = 0;
			out.append(String.valueOf(pem_array[b1 >>> 2 & 63]));
			out.append(String.valueOf(pem_array[(b1 << 4 & 48) + (k >>> 4 & 15)]));
			out.append(String.valueOf(eq));
			out.append(String.valueOf(eq));
			return out.toString();
		}
		if (left == 2)
		{
			byte b2 = b[strt];
			byte b4 = b[strt + 1];
			int l = 0;
			out.append(String.valueOf(pem_array[b2 >>> 2 & 63]));
			out.append(String.valueOf(pem_array[(b2 << 4 & 48) + (b4 >>> 4 & 15)]));
			out.append(String.valueOf(pem_array[(b4 << 2 & 60) + (l >>> 6 & 3)]));
			out.append(String.valueOf(eq));
			return out.toString();
		}
		byte b3 = b[strt];
		byte b5 = b[strt + 1];
		byte b6 = b[strt + 2];
		out.append(String.valueOf(pem_array[b3 >>> 2 & 63]));
		out.append(String.valueOf(pem_array[(b3 << 4 & 48) + (b5 >>> 4 & 15)]));
		out.append(String.valueOf(pem_array[(b5 << 2 & 60) + (b6 >>> 6 & 3)]));
		out.append(String.valueOf(pem_array[b6 & 63]));
		return out.toString();
	}

	static
	{
	}
}

