/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.jdbc.config.gui;

import java.awt.event.*;
import javax.swing.*;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.protocol.jdbc.config.*;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class PoolConfigGui extends JPanel implements ModelSupported,KeyListener
{
	private static String CONNECTIONS = "connections";
	private static String MAXUSE = "maxuse";

	private PoolConfig model;
	private boolean displayName;

	private NamePanel namePanel;
	private JTextField connField = new JTextField(5);
	private JTextField maxUseField = new JTextField(5);

	public PoolConfigGui()
	{
		displayName = true;
	}

	public PoolConfigGui(boolean displayName)
	{
		this.displayName = displayName;
	}

	public void updateGui()
	{
		connField.setText(Integer.toString(model.getNumConnections()));
		maxUseField.setText(Integer.toString(model.getMaxUse()));
	}

	public void setModel(Object model)
	{
		this.model = (PoolConfig)model;
		init();
	}

	private void init()
	{
		System.out.println("Initing poolconfiggui");
		this.setLayout(new VerticalLayout(1,VerticalLayout.LEFT));
		namePanel = new NamePanel(model);
		if(displayName)
		{
			this.add(namePanel);
		}
		this.add(createConnPanel());
		this.add(createMaxUsePanel());
	}

	private JPanel createConnPanel()
	{
		JPanel panel = new JPanel();
		System.out.println("Creating connection panel");
		panel.add(new JLabel("Number of Connections in Pool"));
		connField.setText(Integer.toString(model.getNumConnections()));
		connField.setName(CONNECTIONS);
		connField.addKeyListener(this);
		panel.add(connField);
		System.out.println("Panel = "+panel);
		return panel;
	}

	private JPanel createMaxUsePanel()
	{
		JPanel panel = new JPanel();
		panel.add(new JLabel("Max usage for each connection"));
		maxUseField.setText(Integer.toString(model.getMaxUse()));
		maxUseField.setName(MAXUSE);
		maxUseField.addKeyListener(this);
		panel.add(maxUseField);
		return panel;
	}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyTyped(KeyEvent e)
	{
	}

	public void keyReleased(KeyEvent e)
	{
		String name = e.getComponent().getName();
		if(name.equals(CONNECTIONS))
		{
			model.setNumConnections(Integer.parseInt(connField.getText()));
		}
		else if(name.equals(MAXUSE))
		{
			model.setMaxUse(Integer.parseInt(maxUseField.getText()));
		}
	}
}