/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.timers.gui;

import javax.swing.*;
import java.awt.event.*;

import org.apache.jmeter.timers.*;
import org.apache.jmeter.gui.*;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    $Date: 2001/03/17 22:25:58 $
 *@version    1.0
 ***********************************************************/

public class GaussianRandomTimerGui extends JPanel implements ModelSupported, KeyListener
{

	RandomTimer model;
	private JTextField delayField = new JTextField(6);
	private JTextField rangeField = new JTextField(6);
	private JTextField nameField = new JTextField(30);

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public GaussianRandomTimerGui()
	{
		init();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  model  !ToDo (Parameter description)
	 ***********************************************************/
	public void setModel(Object model)
	{
		this.model = (RandomTimer)model;
		updateGui();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  e  !ToDo (Parameter description)
	 ***********************************************************/
	public void keyReleased(KeyEvent e)
	{
		String temp = e.getComponent().getName();
		if (temp.equals(NAME_FIELD))
		{
			model.setName(nameField.getText());
		}
		else if (temp.equals("Timer Delay"))
		{
			model.setDelay(Long.parseLong(delayField.getText()));
		}
		else if (temp.equals("Timer Delay Range"))
		{
			model.setRange(Double.parseDouble(rangeField.getText()));
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  e  !ToDo (Parameter description)
	 ***********************************************************/
	public void keyPressed(KeyEvent e)
	{
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  e  !ToDo (Parameter description)
	 ***********************************************************/
	public void keyTyped(KeyEvent e)
	{
	}

	public void updateGui()
	{
		delayField.setText("" + model.getDelay());
		rangeField.setText(""+model.getRange());
		nameField.setText(model.getName());
	}

	private void init()
	{
		this.setLayout(new VerticalLayout(VerticalLayout.LEFT));
		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Name"));
		namePanel.add(nameField);
		nameField.addKeyListener(this);

		JLabel delayLabel = new JLabel("Average Delay");
		JLabel rangeLabel = new JLabel("Delay Deviation");

		nameField.setName(NAME_FIELD);
		delayField.setName("Timer Delay");
		rangeField.setName("Timer Delay Range");
		delayField.addKeyListener(this);
		rangeField.addKeyListener(this);
		JPanel delayPane = new JPanel();
		delayPane.add(delayLabel);
		delayPane.add(delayField);

		JPanel rangePane = new JPanel();
		rangePane.add(rangeLabel);
		rangePane.add(rangeField);

		this.add(namePanel);
		this.add(delayPane);
		this.add(rangePane);
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  e        !ToDo (Parameter description)
	 *@param  thrower  !ToDo (Parameter description)
	 ***********************************************************/
	public static void error(Exception e, JComponent thrower)
	{
		JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
	}
}
